@@ -19,6 +19,7 @@ from __future__ import unicode_literals
 
 # import stdlib
 import re
+import logging
 import collections
 from copy import deepcopy
 
@@ -47,6 +48,8 @@ from napalm_base.exceptions import CommandTimeoutException
 # import local modules
 from napalm_junos.utils import junos_views
 
+log = logging.getLogger(__file__)
+
 
 class JunOSDriver(NetworkDriver):
     """JunOSDriver class - inherits NetworkDriver from napalm_base."""
@@ -428,8 +431,15 @@ class JunOSDriver(NetworkDriver):
     def get_lldp_neighbors(self):
         """Return LLDP neighbors details."""
         lldp = junos_views.junos_lldp_table(self.device)
-        lldp.get()
-
+        try:
+            lldp.get()
+        except RpcError as rpcerr:
+            # this assumes the library runs in an environment
+            # able to handle logs
+            # otherwise, the user just won't see this happening
+            log.error('Unable to retrieve the LLDP neighbors information:')
+            log.error(rpcerr.message)
+            return {}
         result = lldp.items()
 
         neighbors = {}
@@ -445,7 +455,15 @@ class JunOSDriver(NetworkDriver):
         lldp_neighbors = {}
 
         lldp_table = junos_views.junos_lldp_neighbors_detail_table(self.device)
-        lldp_table.get()
+        try:
+            lldp_table.get()
+        except RpcError as rpcerr:
+            # this assumes the library runs in an environment
+            # able to handle logs
+            # otherwise, the user just won't see this happening
+            log.error('Unable to retrieve the LLDP neighbors information:')
+            log.error(rpcerr.message)
+            return {}
         interfaces = lldp_table.get().keys()
 
         old_junos = napalm_base.helpers.convert(
@@ -907,19 +925,25 @@ class JunOSDriver(NetworkDriver):
             'inet6': u'ipv6'
             # can add more mappings
         }
+        _FAMILY_MAX_PREFIXLEN = {
+            'inet': 32,
+            'inet6': 128
+        }
 
         for interface_details in interface_table_items:
             ip_network = interface_details[0]
             ip_address = ip_network.split('/')[0]
             address = napalm_base.helpers.convert(
                 napalm_base.helpers.ip, ip_address, ip_address)
-            prefix = napalm_base.helpers.convert(int, ip_network.split('/')[-1], 0)
             try:
                 interface_details_dict = dict(interface_details[1])
                 family_raw = interface_details_dict.get('family')
                 interface = py23_compat.text_type(interface_details_dict.get('interface'))
             except ValueError:
                 continue
+            prefix = napalm_base.helpers.convert(int,
+                                                 ip_network.split('/')[-1],
+                                                 _FAMILY_MAX_PREFIXLEN.get(family_raw))
             family = _FAMILY_VMAP_.get(family_raw)
             if not family or not interface:
                 continue
@@ -1195,7 +1219,8 @@ class JunOSDriver(NetworkDriver):
                    destination,
                    source=C.TRACEROUTE_SOURCE,
                    ttl=C.TRACEROUTE_TTL,
-                   timeout=C.TRACEROUTE_TIMEOUT):
+                   timeout=C.TRACEROUTE_TIMEOUT,
+                   vrf=C.TRACEROUTE_VRF):
         """Execute traceroute and return results."""
         traceroute_result = {}
 
@@ -1205,19 +1230,23 @@ class JunOSDriver(NetworkDriver):
         source_str = ''
         maxttl_str = ''
         wait_str = ''
+        vrf_str = ''
 
         if source:
-            source_str = 'source {source}'.format(source=source)
+            source_str = ' source {source}'.format(source=source)
         if ttl:
-            maxttl_str = 'ttl {ttl}'.format(ttl=ttl)
+            maxttl_str = ' ttl {ttl}'.format(ttl=ttl)
         if timeout:
-            wait_str = 'wait {timeout}'.format(timeout=timeout)
+            wait_str = ' wait {timeout}'.format(timeout=timeout)
+        if vrf:
+            vrf_str = ' routing-instance {vrf}'.format(vrf=vrf)
 
-        traceroute_command = 'traceroute {destination} {source} {maxttl} {wait}'.format(
+        traceroute_command = 'traceroute {destination}{source}{maxttl}{wait}{vrf}'.format(
             destination=destination,
             source=source_str,
             maxttl=maxttl_str,
-            wait=wait_str
+            wait=wait_str,
+            vrf=vrf_str
         )
 
         traceroute_rpc = E('command', traceroute_command)
@@ -1257,7 +1286,7 @@ class JunOSDriver(NetworkDriver):
         return traceroute_result
 
     def ping(self, destination, source=C.PING_SOURCE, ttl=C.PING_TTL,
-             timeout=C.PING_TIMEOUT, size=C.PING_SIZE, count=C.PING_COUNT):
+             timeout=C.PING_TIMEOUT, size=C.PING_SIZE, count=C.PING_COUNT, vrf=C.PING_VRF):
 
         ping_dict = {}
 
@@ -1266,25 +1295,29 @@ class JunOSDriver(NetworkDriver):
         timeout_str = ''
         size_str = ''
         count_str = ''
+        vrf_str = ''
 
         if source:
-            source_str = 'source {source}'.format(source=source)
+            source_str = ' source {source}'.format(source=source)
         if ttl:
-            maxttl_str = 'ttl {ttl}'.format(ttl=ttl)
+            maxttl_str = ' ttl {ttl}'.format(ttl=ttl)
         if timeout:
-            timeout_str = 'wait {timeout}'.format(timeout=timeout)
+            timeout_str = ' wait {timeout}'.format(timeout=timeout)
         if size:
-            size_str = 'size {size}'.format(size=size)
+            size_str = ' size {size}'.format(size=size)
         if count:
-            count_str = 'count {count}'.format(count=count)
+            count_str = ' count {count}'.format(count=count)
+        if vrf:
+            vrf_str = ' routing-instance {vrf}'.format(vrf=vrf)
 
-        ping_command = 'ping {destination} {source} {ttl} {timeout} {size} {count}'.format(
+        ping_command = 'ping {destination}{source}{ttl}{timeout}{size}{count}{vrf}'.format(
             destination=destination,
             source=source_str,
             ttl=maxttl_str,
             timeout=timeout_str,
             size=size_str,
-            count=count_str
+            count=count_str,
+            vrf=vrf_str
         )
 
         ping_rpc = E('command', ping_command)
@@ -1418,7 +1451,8 @@ class JunOSDriver(NetworkDriver):
                                 'input_power': {
                                     'instant': (
                                         float(optics['input_power'])
-                                        if optics['input_power'] != '- Inf'
+                                        if optics['input_power'] not in
+                                        [None, C.OPTICS_NULL_LEVEL]
                                         else 0.0),
                                     'avg': 0.0,
                                     'max': 0.0,
@@ -1427,7 +1461,8 @@ class JunOSDriver(NetworkDriver):
                                 'output_power': {
                                     'instant': (
                                         float(optics['output_power'])
-                                        if optics['output_power'] != '- Inf'
+                                        if optics['output_power'] not in
+                                        [None, C.OPTICS_NULL_LEVEL]
                                         else 0.0),
                                     'avg': 0.0,
                                     'max': 0.0,
@@ -1436,7 +1471,8 @@ class JunOSDriver(NetworkDriver):
                                 'laser_bias_current': {
                                     'instant': (
                                         float(optics['laser_bias_current'])
-                                        if optics['laser_bias_current'] != '- Inf'
+                                        if optics['laser_bias_current'] not in
+                                        [None, C.OPTICS_NULL_LEVEL]
                                         else 0.0),
                                     'avg': 0.0,
                                     'max': 0.0,
@@ -1491,6 +1527,8 @@ class JunOSDriver(NetworkDriver):
                 ri_type = 'default'
             ri_rd = ri_details['route_distinguisher']
             ri_interfaces = ri_details['interfaces']
+            if not isinstance(ri_interfaces, list):
+                ri_interfaces = [ri_interfaces]
             network_instances[ri_name] = {
                 'name': ri_name,
                 'type': C.OC_NETWORK_INSTANCE_TYPE_MAP.get(ri_type, ri_type),  # default: return raw
